/**
 * Test case for create.
 * Runs with mocha.
 */
'use strict'

const create = require('../lib/create.js')
const apemanApi = require('apeman-api')
const apemanApiClient = require('apeman-api/client')
const asleep = require('asleep')
const aport = require('aport')
const aschema = require('aschema')
const assert = require('assert')
const co = require('co')

describe('create', function () {
  this.timeout(3000)
  let port, baseUrl
  before(() => co(function * () {
    port = yield aport()
    baseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
  }))

  it('Create', () => co(function * () {
    let heySchema = aschema({
      type: 'object',
      properties: {
        message: {
          type: 'string'
        }
      },
      required: [ 'message' ]
    })
    let server = yield apemanApi({
      port,
      configuration: {
        $api: {
          pingPong: {
            ping () {
              return 'pong'
            }
          },
          foo: {
            sayHey (message) {
              const ctx = this
              ctx.validate(heySchema, { message })
              console.log(message)
            }
          },
          $middlewares: [
            create({})
          ],
          $pathname: '/api/rpc'
        }
      }

    })
    assert(server)

    let client = apemanApiClient(`${baseUrl}/api/rpc`)
    assert.ok(client)

    {
      let pingPong = yield client.connect('pingPong')
      let pong = yield pingPong.ping()
      assert.equal(pong, 'pong')
    }

    {
      let foo = yield client.connect('foo')
      try {
        yield foo.sayHey()
      } catch (e) {
        assert.equal(e.status, 400)
      }

      yield foo.sayHey('hey!')
    }

    yield asleep(300)
    yield server.close()
  }))
})

/* global describe, before, after, it */
